@@ -1,4 +1,4 @@
-# Copyrights 2006-2014 by [Mark Overmeer].
+# Copyrights 2006-2015 by [Mark Overmeer].
 #  For other contributors see ChangeLog.
 # See the manual pages for details on the licensing terms.
 # Pod stripped from pm file by OODoc 2.01.
@@ -8,7 +8,7 @@ use strict;
 
 package XML::Compile;
 use vars '$VERSION';
-$VERSION = '1.47';
+$VERSION = '1.50';
 
 
 use Log::Report 'xml-compile';
@@ -33,7 +33,7 @@ __PACKAGE__->addSchemaDirs($ENV{SCHEMA_DIRECTORIES});
 __PACKAGE__->addSchemaDirs(__FILE__);
 
 
-sub new($@)
+sub new(@)
 {   my $class = shift;
     my $top   = @_ % 2 ? shift : undef;
 
@@ -128,7 +128,9 @@ sub dataToXML($)
     }
     else
     {   my $data = "$raw";
-        $data = substr($data, 0, 59) . '...' if length($data) > 60;
+        $data = substr($data, 0, 59) . '...'
+            if length($data) > 60 && $data =~ m/\</;
+
         error __x"don't known how to interpret XML data\n   {data}"
            , data => $data;
     }
