@@ -1,10 +1,10 @@
-# Copyrights 2006-2014 by [Mark Overmeer].
+# Copyrights 2006-2015 by [Mark Overmeer].
 #  For other contributors see ChangeLog.
 # See the manual pages for details on the licensing terms.
 # Pod stripped from pm file by OODoc 2.01.
 package XML::Compile::Translate::Reader;
 use vars '$VERSION';
-$VERSION = '1.47';
+$VERSION = '1.50';
 
 use base 'XML::Compile::Translate';
 
@@ -14,7 +14,7 @@ no warnings 'once', 'recursion';
 
 use Log::Report 'xml-compile', syntax => 'SHORT';
 use List::Util   qw/first/;
-use Scalar::Util qw/weaken/;
+use Scalar::Util qw/weaken blessed/;
 
 use XML::Compile::Util qw/pack_type odd_elements type_of_node SCHEMA2001i/;
 use XML::Compile::Iterator ();
@@ -78,7 +78,7 @@ sub makeElementWrapper
 {   my ($self, $path, $processor) = @_;
     # no copy of $_[0], because it may be a large string
     sub { my $tree;
-          if(ref $_[0] && UNIVERSAL::isa($_[0], 'XML::LibXML::Iterator'))
+          if(blessed $_[0] && $_[0]->isa('XML::LibXML::Iterator'))
           {   $tree = $_[0];
           }
           else
@@ -139,6 +139,7 @@ sub makeSequence($@)
     sub { my $tree = shift;
           my @res;
           my @do = @pairs;
+
           while(@do)
           {   my ($take, $do) = (shift @do, shift @do);
               push @res, ref $do eq 'BLOCK'
@@ -224,8 +225,9 @@ sub makeChoice($@)
               or error __x"choice needs more elements at {path}"
                    , path => $path, _class => 'misfit';
 
+
           my @elems = sort keys %do;
-          trace "choose element from @elems or fix special" if @elems;
+          trace "choose element from @elems or fix special at $path" if @elems;
           trace "failed specials in choice: $_" for @special_errors;
 
           error __x"no applicable choice for `{tag}' at {path}"
@@ -403,7 +405,7 @@ sub makeBlockHandler
 
 sub makeElementHandler
 {   my ($self, $path, $label, $min, $max, $required, $optional) = @_;
-    $max eq "0" and return sub {};  # max can be "unbounded"
+    $max eq "0" and return sub {};  # max can be "unbounded", hence strcmp
 
     if($max ne 'unbounded' && $max==1)
     {   return $min==1
@@ -485,6 +487,7 @@ sub makeElementHref
 {   my ($self, $path, $ns, $childname, $do) = @_;
 
     sub { my $tree  = shift;
+
           return ($childname => $tree->node)
               if defined $tree
               && $tree->nodeType eq $childname
@@ -587,7 +590,7 @@ sub _not_processed($$)
 
 sub makeComplexElement
 {   my ($self, $path, $tag, $elems, $attrs, $attrs_any,undef,$is_nillable) = @_;
-my @e = @$elems; my @a = @$attrs;
+#my @e = @$elems; my @a = @$attrs;
 
     my @elems = odd_elements @$elems;
     my @attrs = (odd_elements(@$attrs), @$attrs_any);
@@ -930,7 +933,7 @@ sub makeSubstgroup
     sub { my $tree  = shift;
           my $type  = ($tree ? $tree->currentType : undef)
               or error __x"no data for substitution group {type} at {path}"
-                    , type => $base, path => $path;
+                    , type => $base, path => $path, class => 'misfit';
 
           my $do    = $do{$type} or return ();
           my @subst = $do->[1]($tree->descend);
@@ -1085,8 +1088,8 @@ sub makeXsiTypeSwitch($$$$)
 
 # any kind of hook
 
-sub makeHook($$$$$$)
-{   my ($self, $path, $r, $tag, $before, $replace, $after) = @_;
+sub makeHook($$$$$$$)
+{   my ($self, $path, $r, $tag, $before, $replace, $after, $fulltype) = @_;
     return $r unless $before || $replace || $after;
 
     return sub { ($_[0]->node->localName => 'SKIPPED') }
@@ -1102,19 +1105,19 @@ sub makeHook($$$$$$)
      { my $tree = shift or return ();
        my $xml  = $tree->node;
        foreach (@before)
-       {   $xml = $_->($xml, $path);
+       {   $xml = $_->($xml, $path, $fulltype);
            defined $xml or return ();
        }
 
        my $process = sub { $r->($tree->descend($xml)) };
        my @h = @replace
-         ? map $_->($xml, $self, $path, $tag, $process), @replace
+         ? map $_->($xml, $self, $path, $tag, $process, $fulltype), @replace
          : $process->();
 
        @h or return ();
-       my $h = @h==1 && !ref $h[0] ? {_ => $h[0]} : $h[1];  # detect simpleType
+       my $h = @h==1 ? $h[0] : $h[1];  # detect simpleType
        foreach my $after (@after)
-       {   $h = $after->($xml, $h, $path);
+       {   $h = $after->($xml, $h, $path, $fulltype);
            defined $h or return ();
        }
        ($tag => $h);
